@@ -32,6 +32,7 @@ typedef struct {
 	int bufl;
 	int line;
 	int err;
+	int ms_warn;
 	char *cmd, *val, *rest;
 } conffile_t;
 
@@ -40,11 +41,11 @@ typedef struct {
 
 char *get_arg( conffile_t *cfile, int required, int *comment );
 
-int parse_bool( conffile_t *cfile );
+char parse_bool( conffile_t *cfile );
 int parse_int( conffile_t *cfile );
-int parse_size( conffile_t *cfile );
+uint parse_size( conffile_t *cfile );
 int getcline( conffile_t *cfile );
 int merge_ops( int cops, int ops[] );
-int load_config( const char *filename, int pseudo );
+int load_config( const char *filename );
 
 #endif
